// src/lib/mysql.js
import dotenv from "dotenv";
import mysql from "mysql2/promise";
dotenv.config();
var MYSQL_DATABASE = process.env.MYSQL_DATABASE;
var MYSQL_USER = process.env.MYSQL_USER;
var MYSQL_PASSWORD = process.env.MYSQL_PASSWORD;
var MYSQL_HOST = process.env.MYSQL_HOST;
var MYSQL_PORT = process.env.MYSQL_PORT;
var MySQL = class _MySQL {
  static instance;
  pool = null;
  constructor() {
    if (!_MySQL.instance) {
      try {
        this.pool = mysql.createPool({
          host: MYSQL_HOST,
          user: MYSQL_USER,
          password: MYSQL_PASSWORD,
          database: MYSQL_DATABASE,
          port: Number(MYSQL_PORT)
        });
        _MySQL.instance = this;
      } catch (error) {
        console.error("Error al conectar con MySQL:", error);
      }
    }
    return _MySQL.instance;
  }
  /**
   * Creates and returns a new instance of `TableQuery` for the specified table.
   * This method is used to start building queries for a specific table.
   *
   * @param {string} tableName - The name of the table to query.
   * @returns {TableQuery} - Returns a new instance of `TableQuery` for the specified table.
   *
   * @example
   * const usersTable = db.table('users');
   * const users = await usersTable.select(['id', 'name']).get();
   * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
   */
  table(tableName) {
    return new TableQuery(tableName, this.pool);
  }
  /**
   * Executes a SQL query on the currently set database.
   *
   * @param {string} sqlQuery - The SQL query to execute.
   * @returns {Promise<{ status: string, message: string, data: any | null }>} - Returns a JSON object with the status, message, and data (if any).
   *
   * @example
   * const result = await db.query('SELECT * FROM users;');
   * console.log(result);
   * // { status: 'success', message: 'Query executed successfully', data: [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }] }
   *
   * @example
   * const result = await db.query('INVALID SQL QUERY;');
   * console.log(result);
   * // { status: 'error', message: 'SQL syntax error', data: null }
   */
  async query(sqlQuery) {
    if (typeof sqlQuery !== "string") {
      throw new Error("The SQL query must be a string.");
    }
    const sqlCommands = sqlQuery.split(";").filter((cmd) => cmd.trim().length > 0);
    try {
      if (!this.pool) {
        throw new Error("Database connection pool is not available.");
      }
      let results = [];
      for (const command of sqlCommands) {
        const query = `${command};`;
        const [result] = await this.pool.query(query);
        results.push(result);
      }
      return {
        status: "success",
        message: "Query executed successfully",
        data: results.length === 1 ? results[0] : results
        // Si hay un solo comando, devolver solo ese resultado
      };
    } catch (error) {
      return {
        status: "error",
        message: error.sqlMessage || error.message || "An error occurred while executing the query.",
        data: null
      };
    }
  }
};
var TableQuery = class _TableQuery {
  conection;
  // Tipo de conexión, puedes reemplazar `any` con el tipo adecuado
  nextType;
  // Almacenar el tipo para la próxima condición
  joins;
  // Almacenar los JOINs
  _orderBy;
  // Almacenar los ORDER BY
  _distinct;
  // Para controlar si se utiliza DISTINCT
  _groupBy;
  // Almacenar los GROUP BY
  tableName;
  // Nombre de la tabla
  fields;
  // Campos seleccionados
  query;
  // Consulta SQL construida
  conditions;
  // Condiciones WHERE
  limitValue = null;
  // Límite de resultados
  pageValue = null;
  // Límite de resultados
  constructor(tableName, conection = null) {
    this.tableName = tableName;
    this.fields = [];
    this.nextType = "AND";
    this.joins = [];
    this.query = `SELECT * FROM \`${tableName}\``;
    this.conditions = [];
    this._distinct = false;
    this._orderBy = [];
    this._groupBy = [];
    this.conection = conection;
  }
  columns() {
    return new Columns(this.tableName, this.conection);
  }
  async create(fields) {
    try {
      const fieldsDefinition = fields.map((field) => {
        const { name, type, defaultValue, length, options, foreing } = field;
        if (!name || !type) {
          throw new Error("Cada campo debe tener un nombre y un tipo.");
        }
        let fieldDefinition = length && type != "text" ? `\`${name}\` ${type}(${length})` : `\`${name}\` ${type}`;
        if (defaultValue) {
          fieldDefinition += ["VARCHAR", "CHAR", "TEXT", "ENUM", "SET"].includes(type.toUpperCase()) ? defaultValue ? ` DEFAULT '${defaultValue}'` : ` DEFAULT NULL` : defaultValue === "NONE" || defaultValue === null ? "" : defaultValue ? ` DEFAULT ${defaultValue}` : ` DEFAULT NULL`;
        }
        if (options) {
          if (options.includes("primary")) {
            fieldDefinition += " PRIMARY KEY";
          }
          if (options.includes("autoincrement")) {
            fieldDefinition += " AUTO_INCREMENT";
          }
          if (options.includes("unique")) {
            fieldDefinition += " UNIQUE";
          }
        }
        if (foreing) {
          fieldDefinition += `, FOREIGN KEY (\`${name}\`) REFERENCES \`${foreing.table}\`(\`${foreing.column}\`)`;
        }
        return fieldDefinition;
      }).join(", ");
      let sqlQuery = `CREATE TABLE IF NOT EXISTS \`${this.tableName}\` (${fieldsDefinition}`;
      sqlQuery += ")";
      await this.#get_response(sqlQuery);
      return true;
    } catch (error) {
      throw error;
    }
  }
  async drop() {
    try {
      const sqlQuery = `DROP TABLE IF EXISTS \`${this.tableName}\``;
      await this.#get_response(sqlQuery);
      return true;
    } catch (error) {
      throw new Error("Error al eliminar la tabla: " + error.message);
    }
  }
  /**
   * Specifies the columns to select in a SELECT query.
   *
   * @param {string[]} fields - Array of column names to select. If empty, selects all columns.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').select(['id', 'name']).get();
   * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
   */
  select(fields = []) {
    if (fields.length > 0) {
      this.query = `SELECT ${this._distinct ? "DISTINCT " : ""}${fields.join(", ")} FROM \`${this.tableName}\``;
    }
    return this;
  }
  /**
   * Adds a WHERE condition to the query.
   *
   * @param {string} column - The column to filter by.
   * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
   * @param {any} value - The value to compare against.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').where('age', '>', 25).get();
   * console.log(users); // [{ id: 1, name: 'John', age: 30 }]
   */
  where(column, operator, value) {
    if (operator === void 0) {
      operator = "=";
    }
    this.conditions.push({ column, operator, value, type: this.nextType, isGroup: false });
    this.nextType = "AND";
    return this;
  }
  /**
   * Adds an OR WHERE condition to the query.
   *
   * @param {string} column - The column to filter by.
   * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
   * @param {any} value - The value to compare against.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').where('age', '>', 25).orWhere('name', '=', 'Jane').get();
   * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
   */
  orWhere(column, operator, value) {
    if (operator === void 0) {
      operator = "=";
    }
    this.conditions.push({ column, operator, value, type: "OR", isGroup: false });
    return this;
  }
  /**
   * Adds a grouped WHERE condition to the query.
   *
   * @param {Function} callback - A callback function that receives a new TableQuery instance to build the grouped conditions.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').whereGroup(query => {
   *     query.where('age', '>', 25).orWhere('name', '=', 'Jane');
   * }).get();
   * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
   */
  whereGroup(callback) {
    const groupQuery = new _TableQuery(this.tableName);
    callback(groupQuery);
    const groupConditions = groupQuery.buildConditions();
    this.conditions.push({ query: groupConditions, type: this.nextType, isGroup: true });
    this.nextType = "AND";
    return this;
  }
  or() {
    this.nextType = "OR";
    return this;
  }
  and() {
    this.nextType = "AND";
    return this;
  }
  /**
   * Adds a WHERE BETWEEN condition to the query.
   *
   * @param {string} column - The column to filter by.
   * @param {Array<any>} values - An array with two values representing the range.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').whereBetween('age', [20, 30]).get();
   * console.log(users); // [{ id: 1, name: 'John', age: 30 }, { id: 2, name: 'Jane', age: 25 }]
   */
  whereBetween(column, [value1, value2]) {
    if (Array.isArray([value1, value2]) && value1 !== void 0 && value2 !== void 0) {
      this.conditions.push({ column, operator: "BETWEEN", value: [value1, value2], type: this.nextType, isGroup: false });
      this.nextType = "AND";
    }
    return this;
  }
  /**
   * Adds a WHERE IN condition to the query.
   *
   * @param {string} column - The column to filter by.
   * @param {Array<any>} values - An array of values to match.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').whereIn('id', [1, 2]).get();
   * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
   */
  whereIn(column, values) {
    if (Array.isArray(values) && values.length > 0) {
      this.conditions.push({ column, operator: "IN", value: values, type: this.nextType, isGroup: false });
      this.nextType = "AND";
    }
    return this;
  }
  /**
  * Adds a WHERE IS NULL condition to the query.
  *
  * @param {string} column - The column to filter by.
  * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
  *
  * @example
  * const users = await db.table('users').whereNull('email').get();
  * console.log(users); // [{ id: 3, name: 'Alice', email: null }]
  */
  whereNull(column) {
    this.conditions.push({ column, operator: "IS NULL", type: this.nextType, isGroup: false });
    this.nextType = "AND";
    return this;
  }
  /**
   * Adds a WHERE IS NOT NULL condition to the query.
   *
   * @param {string} column - The column to filter by.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').whereNotNull('email').get();
   * console.log(users); // [{ id: 1, name: 'John', email: 'john@example.com' }]
   */
  whereNotNull(column) {
    this.conditions.push({ column, operator: "IS NOT NULL", type: this.nextType, isGroup: false });
    this.nextType = "AND";
    return this;
  }
  buildQuery(includeSelect = true) {
    let query = includeSelect ? this.query : "";
    if (this.joins.length > 0) {
      query += ` ${this.joins.join(" ")}`;
    }
    const whereClauses = this.buildConditions();
    if (whereClauses.length > 0) {
      query += ` WHERE ${whereClauses}`;
    }
    if (this._groupBy.length > 0) {
      query += ` GROUP BY ${this._groupBy.join(", ")}`;
    }
    if (this.limitValue !== null && this.limitValue !== void 0 && !Number.isNaN(this.limitValue)) {
      query += ` LIMIT ${this.limitValue}`;
    }
    if (this.limitValue && this.pageValue !== null && this.pageValue !== void 0 && !Number.isNaN(this.pageValue)) {
      const offset = (this.pageValue - 1) * this.limitValue;
      query += ` OFFSET ${offset}`;
    }
    if (this._orderBy.length > 0 && !this.query.startsWith("SELECT COUNT") && !this.query.startsWith("SELECT SUM") && !this.query.startsWith("SELECT AVG") && !this.query.startsWith("SELECT MAX") && !this.query.startsWith("SELECT MIN")) {
      const orderByClauses = this._orderBy.map((order) => `${order.column} ${order.direction}`).join(", ");
      query += ` ORDER BY ${orderByClauses}`;
    }
    return query;
  }
  buildConditions() {
    return this.conditions.map((cond, index) => {
      const prefix = index === 0 ? "" : ` ${cond.type} `;
      if (cond.isGroup) {
        return `${prefix}(${cond.query})`;
      }
      let conditionStr = "";
      if (cond.operator === "BETWEEN") {
        const [value1, value2] = cond.value;
        const formattedValue1 = typeof value1 === "string" ? `'${value1}'` : value1;
        const formattedValue2 = typeof value2 === "string" ? `'${value2}'` : value2;
        conditionStr = `${cond.column} BETWEEN ${formattedValue1} AND ${formattedValue2}`;
      } else if (cond.operator === "IN") {
        const values = cond.value.map((val) => typeof val === "string" ? `'${val}'` : val).join(", ");
        conditionStr = `${cond.column} IN (${values})`;
      } else if (cond.operator === "IS NULL") {
        conditionStr = `${cond.column} IS NULL`;
      } else if (cond.operator === "IS NOT NULL") {
        conditionStr = `${cond.column} IS NOT NULL`;
      } else {
        const value = typeof cond.value === "string" ? `'${cond.value}'` : cond.value;
        conditionStr = `${cond.column} ${cond.operator} ${value}`;
      }
      return `${prefix}${conditionStr}`;
    }).join("");
  }
  /**
   * Adds a JOIN clause to the query.
   *
   * @param {string} table - The table to join.
   * @param {string} column1 - The column from the current table.
   * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
   * @param {string} column2 - The column from the joined table.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').join('orders', 'users.id', '=', 'orders.user_id').get();
   * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }]
   */
  join(table, column1, operator, column2) {
    this.joins.push(`JOIN ${table} ON ${column1} ${operator} ${column2}`);
    return this;
  }
  /**
   * Adds a LEFT JOIN clause to the query.
   *
   * @param {string} table - The table to join.
   * @param {string} column1 - The column from the current table.
   * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
   * @param {string} column2 - The column from the joined table.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').leftJoin('orders', 'users.id', '=', 'orders.user_id').get();
   * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }, { id: 2, name: 'Jane', order_id: null }]
   */
  leftJoin(table, column1, operator, column2) {
    this.joins.push(`LEFT JOIN ${table} ON ${column1} ${operator} ${column2}`);
    return this;
  }
  /**
   * Adds a RIGHT JOIN clause to the query.
   *
   * @param {string} table - The table to join.
   * @param {string} column1 - The column from the current table.
   * @param {string} operator - The comparison operator (e.g., '=', '>', '<').
   * @param {string} column2 - The column from the joined table.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').rightJoin('orders', 'users.id', '=', 'orders.user_id').get();
   * console.log(users); // [{ id: 1, name: 'John', order_id: 101 }, { id: null, name: null, order_id: 102 }]
   */
  rightJoin(table, column1, operator, column2) {
    this.joins.push(`RIGHT JOIN ${table} ON ${column1} ${operator} ${column2}`);
    return this;
  }
  /**
   * Adds an ORDER BY clause to the query.
   *
   * @param {string} column - The column to order by.
   * @param {string} direction - The sorting direction ('ASC' or 'DESC').
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').orderBy('name', 'ASC').get();
   * console.log(users); // [{ id: 2, name: 'Jane' }, { id: 1, name: 'John' }]
   */
  orderBy(column, direction = "ASC") {
    const validDirections = ["ASC", "DESC"];
    if (validDirections.includes(direction.toUpperCase())) {
      this._orderBy.push({ column, direction: direction.toUpperCase() });
    } else {
      throw new Error(`Invalid direction: ${direction}. Use 'ASC' or 'DESC'.`);
    }
    return this;
  }
  /**
   * Adds a GROUP BY clause to the query.
   *
   * @param {string} column - The column to group by.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').groupBy('age').get();
   * console.log(users); // [{ age: 30, count: 1 }, { age: 25, count: 1 }]
   */
  groupBy(column) {
    this._groupBy.push(column);
    return this;
  }
  /**
   * Adds a DISTINCT clause to the query.
   *
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').distinct().select(['name']).get();
   * console.log(users); // [{ name: 'John' }, { name: 'Jane' }]
   */
  distinct() {
    this._distinct = true;
    this.query = this.query.replace(/^SELECT /, "SELECT DISTINCT ");
    return this;
  }
  /**
   * Adds a COUNT clause to the query.
   *
   * @param {string} column - The column to count (default is '*').
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const count = await db.table('users').count().first();
   * console.log(count); // { count: 2 }
   */
  count(column = "*") {
    this.query = `SELECT COUNT(${column}) AS count FROM ${this.tableName}`;
    return this;
  }
  /**
   * Adds a SUM clause to the query.
   *
   * @param {string} column - The column to sum.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const totalAge = await db.table('users').sum('age').first();
   * console.log(totalAge); // { sum: 55 }
   */
  sum(column) {
    this.query = `SELECT SUM(${column}) AS sum FROM \`${this.tableName}\``;
    return this;
  }
  /**
   * Adds an AVG clause to the query.
   *
   * @param {string} column - The column to calculate the average.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const avgAge = await db.table('users').avg('age').first();
   * console.log(avgAge); // { avg: 27.5 }
   */
  avg(column) {
    this.query = `SELECT AVG(${column}) AS avg FROM \`${this.tableName}\``;
    return this;
  }
  /**
   * Adds a MAX clause to the query.
   *
   * @param {string} column - The column to find the maximum value.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const maxAge = await db.table('users').max('age').first();
   * console.log(maxAge); // { max: 30 }
   */
  max(column) {
    this.query = `SELECT MAX(${column}) AS max FROM \`${this.tableName}\``;
    return this;
  }
  /**
   * Adds a MIN clause to the query.
   *
   * @param {string} column - The column to find the minimum value.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const minAge = await db.table('users').min('age').first();
   * console.log(minAge); // { min: 25 }
   */
  min(column) {
    this.query = `SELECT MIN(${column}) AS min FROM \`${this.tableName}\``;
    return this;
  }
  /**
   * Adds a LIMIT clause to the query.
   *
   * @param {number} number - The maximum number of rows to return.
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').limit(1).get();
   * console.log(users); // [{ id: 1, name: 'John', age: 30 }]
   */
  limit(number) {
    this.limitValue = number;
    return this;
  }
  /**
   * Adds pagination to the query using LIMIT and OFFSET.
   *
   * @param {number} number - The page number (starting from 1).
   * @returns {TableQuery} - Returns the current instance of TableQuery for method chaining.
   *
   * @example
   * const users = await db.table('users').limit(1).page(2).get();
   * console.log(users); // [{ id: 2, name: 'Jane', age: 25 }]
   */
  page(number) {
    this.pageValue = number;
    return this;
  }
  /**
  * Executes the query and returns all matching rows.
  *
  * @returns {Promise<Array<Object>>} - Returns an array of rows.
  *
  * @example
  * const users = await db.table('users').get();
  * console.log(users); // [{ id: 1, name: 'John' }, { id: 2, name: 'Jane' }]
  */
  async get() {
    const sqlQuery = this.buildQuery();
    try {
      const result = await this.#get_response(sqlQuery);
      return result;
    } catch (error) {
      throw error;
    }
  }
  /**
   * Executes the query and returns the first matching row.
   *
   * @returns {Promise<Object | null>} - Returns the first row or null if no rows match.
   *
   * @example
   * const user = await db.table('users').first();
   * console.log(user); // { id: 1, name: 'John' }
   */
  async first() {
    const sqlQuery = this.buildQuery();
    try {
      const result = await this.#get_response(sqlQuery);
      return result[0] || null;
    } catch (error) {
      throw error;
    }
  }
  /**
   * Finds a row by a specific column value.
   *
   * @param {any} value - The value to search for.
   * @param {string} column - The column to search in (default is 'id').
   * @returns {Promise<Object | null>} - Returns the first matching row or null if no rows match.
   *
   * @example
   * const user = await db.table('users').find(1);
   * console.log(user); // { id: 1, name: 'John' }
   */
  async find(value, column = "id") {
    this.where(column, "=", value);
    const sqlQuery = this.buildQuery();
    try {
      const result = await this.#get_response(sqlQuery);
      return result[0] || null;
    } catch (error) {
      throw error;
    }
  }
  /**
   * Inserts one or more rows into the table.
   *
   * @param {Array<Object>} data - An array of objects representing the rows to insert.
   * @returns {Promise<Array<Object>>} - Returns an array of the inserted rows.
   *
   * @example
   * const newUsers = await db.table('users').insert([
   *     { name: 'Alice', age: 28 },
   *     { name: 'Bob', age: 32 }
   * ]);
   * console.log(newUsers); // [{ id: 3, name: 'Alice', age: 28 }, { id: 4, name: 'Bob', age: 32 }]
   */
  async insert(data) {
    if (!Array.isArray(data)) {
      throw new Error("El m\xE9todo insert requiere un array de objetos con pares clave-valor.");
    }
    if (!data.every((item) => typeof item === "object" && item !== null)) {
      throw new Error("El array debe contener solo objetos v\xE1lidos.");
    }
    try {
      const results = [];
      for (const row of data) {
        const keys = Object.keys(row).map((key) => `\`${key}\``);
        const values = Object.values(row).map((value) => {
          if (value === void 0 || value === null) {
            return "NULL";
          }
          return typeof value === "string" ? `'${value}'` : value;
        });
        const columns = keys.join(", ");
        const placeholders = values.join(", ");
        const sqlQuery = `INSERT INTO \`${this.tableName}\` (${columns}) VALUES (${placeholders})`;
        const result = await this.#get_response(sqlQuery);
        const insertedRow = await this.where("id", "=", result.insertId || 0).first();
        results.push(insertedRow);
      }
      return results;
    } catch (error) {
      throw new Error("Error al insertar los datos: " + error.message);
    }
  }
  /**
   * Inserts one or more rows into the table.
   *
   * @param {Array<Object>} data - An array of objects representing the rows to insert.
   * @returns {Promise<Array<Object>>} - Returns an array of the inserted rows.
   *
   * @example
   * const newUsers = await db.table('users').insert([
   *     { name: 'Alice', age: 28 },
   *     { name: 'Bob', age: 32 }
   * ]);
   * console.log(newUsers); // [{ id: 3, name: 'Alice', age: 28 }, { id: 4, name: 'Bob', age: 32 }]
   */
  async update(data) {
    if (typeof data !== "object" || Array.isArray(data)) {
      throw new Error("El m\xE9todo update requiere un objeto con pares clave-valor.");
    }
    const updates = Object.keys(data).map((key) => {
      const value = data[key];
      return `${key} = ${typeof value === "string" ? `'${value}'` : value}`;
    }).join(", ");
    const whereClauses = this.buildConditions();
    if (whereClauses.length === 0) {
      throw new Error("Debe especificar al menos una condici\xF3n WHERE para realizar un update.");
    }
    const sqlQuery = `UPDATE \`${this.tableName}\` SET ${updates} WHERE ${whereClauses}`;
    try {
      const result = await this.#get_response(sqlQuery);
      return result;
    } catch (error) {
      throw error;
    }
  }
  /**
   * Deletes rows from the table based on the defined conditions.
   *
   * @returns {Promise<Object>} - Returns the result of the delete operation.
   *
   * @example
   * const result = await db.table('users').where('id', '=', 1).delete();
   * console.log(result); // { affectedRows: 1 }
   */
  async delete() {
    const whereClauses = this.buildConditions();
    if (whereClauses.length === 0) {
      throw new Error("Debe especificar al menos una condici\xF3n WHERE para realizar un delete.");
    }
    const sqlQuery = `DELETE FROM \`${this.tableName}\` WHERE ${whereClauses}`;
    try {
      const result = await this.#get_response(sqlQuery);
      return result;
    } catch (error) {
      throw error;
    }
  }
  async #get_response(sql) {
    const pool = await this.conection;
    if (!pool) {
      throw new Error("No se ha establecido una conexi\xF3n a la base de datos.");
    }
    try {
      const [result] = await pool.query(sql);
      return result;
    } catch (error) {
      if (error.code === "ER_BAD_DB_ERROR") {
        try {
          if (!this.conection) {
            throw new Error("No se ha establecido una conexi\xF3n a la base de datos.");
          }
          await this.conection.end();
          this.conection = mysql.createPool({
            host: MYSQL_HOST,
            user: MYSQL_USER,
            password: MYSQL_PASSWORD,
            port: Number(MYSQL_PORT)
          });
          const pool_aux = await this.conection;
          await pool_aux.query(`CREATE DATABASE \`${MYSQL_DATABASE}\``);
          await this.conection.end();
          this.conection = mysql.createPool({
            host: MYSQL_HOST,
            user: MYSQL_USER,
            password: MYSQL_PASSWORD,
            database: MYSQL_DATABASE,
            port: Number(MYSQL_PORT)
          });
          const pool2 = await this.conection;
          const [result] = await pool2.query(sql);
          return result;
        } catch (error2) {
          throw error2;
        }
      }
      throw error;
    }
  }
};
var Columns = class {
  conection;
  tableName;
  // Nombre de la tabla
  constructor(tableName, conection = null) {
    this.tableName = tableName;
    this.conection = conection;
  }
  /**
   * Retrieves the structure of the table columns.
   *
   * @returns {Promise<Object>} - Returns an object with the column details.
   *
   * @example
   * const columns = await db.table('users').columns().get();
   * console.log(columns); // { id: { type: 'INT', defaultValue: null, key: 'PRI', extra: 'auto_increment' } }
   */
  async get() {
    try {
      const tableExistsQuery = `SHOW TABLES LIKE '${this.tableName}'`;
      const tableExistsResult = await this.#get_response(tableExistsQuery);
      if (tableExistsResult && tableExistsResult.length > 0) {
        const existingFieldsQuery = `SHOW COLUMNS FROM \`${this.tableName}\``;
        const existingFields = await this.#get_response(existingFieldsQuery);
        return existingFields.reduce((acc, field) => {
          acc[field.Field] = {
            type: field.Type,
            defaultValue: field.Default,
            key: field.Key,
            extra: field.Extra
          };
          return acc;
        }, {});
      } else {
        return {};
      }
    } catch (error) {
      throw error;
    }
  }
  /**
   * Adds new columns to the table.
   *
   * @param {Array<Field>} fields - An array of objects defining the new columns.
   * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
   *
   * @example
   * await db.table('users').columns().add([
   *     { name: 'email', type: 'VARCHAR', length: 255, defaultValue: 'example@example.com' }
   * ]);
   */
  async add(fields) {
    try {
      const currentFields = await this.get();
      for (const field of fields) {
        const { name, type, length, defaultValue, options, foreing } = field;
        const fullType = length && type !== "TEXT" ? `${type}(${length})` : type;
        if (!currentFields[name]) {
          let alterQuery = `ALTER TABLE \`${this.tableName}\` ADD COLUMN \`${name}\` ${fullType}`;
          if (defaultValue) {
            alterQuery += ["varchar", "char", "text", "enum", "set"].includes(type) ? defaultValue ? ` DEFAULT '${defaultValue}'` : ` DEFAULT NULL` : defaultValue === "NONE" || defaultValue === null ? "" : defaultValue ? ` DEFAULT ${defaultValue}` : ` DEFAULT NULL`;
          }
          if (options) {
            if (options.includes("primary")) {
              alterQuery += " PRIMARY KEY";
            }
            if (options.includes("autoincrement")) {
              alterQuery += " AUTO_INCREMENT";
            }
            if (options.includes("unique")) {
              alterQuery += " UNIQUE";
            }
          }
          if (foreing) {
            alterQuery += `, ADD FOREIGN KEY (\`${name}\`) REFERENCES \`${foreing.table}\`(\`${foreing.column}\`)`;
          }
          await this.#get_response(alterQuery);
        }
      }
      return true;
    } catch (error) {
      console.error("Error al agregar columnas.", error);
      throw error;
    }
  }
  /**
   * Edits existing columns in the table.
   *
   * @param {Array<Field>} fields - An array of objects defining the columns to edit.
   * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
   *
   * @example
   * await db.table('users').columns().edit([
   *     { name: 'email', type: 'VARCHAR', length: 255, defaultValue: 'new@example.com' }
   * ]);
   */
  async edit(fields) {
    try {
      const currentFields = await this.get();
      for (const field of fields) {
        const { name, type, length, defaultValue, options, foreing } = field;
        const fullType = length && type !== "TEXT" ? `${type}(${length})` : type;
        if (currentFields[name]) {
          const existingField = currentFields[name];
          if (existingField.type !== fullType || existingField.defaultValue !== defaultValue || options && options.includes("autoincrement") && existingField.extra !== "auto_increment" || options && options.includes("unique") && existingField.key !== "UNI" || options && options.includes("primary") && existingField.key !== "PRI") {
            let modifyQuery = `ALTER TABLE \`${this.tableName}\` MODIFY COLUMN \`${name}\` ${fullType}`;
            if (existingField.defaultValue !== defaultValue) {
              modifyQuery += ["varchar", "char", "text", "enum", "set"].includes(type) ? defaultValue ? ` DEFAULT '${defaultValue}'` : ` DEFAULT NULL` : defaultValue === "NONE" || defaultValue === null ? "" : defaultValue ? ` DEFAULT ${defaultValue}` : ` DEFAULT NULL`;
            }
            if (options) {
              if (options.includes("primary")) {
                modifyQuery += " PRIMARY KEY";
              }
              if (options.includes("autoincrement")) {
                modifyQuery += " AUTO_INCREMENT";
              }
              if (options.includes("unique")) {
                modifyQuery += " UNIQUE";
              }
            }
            if (foreing) {
              modifyQuery += `, ADD FOREIGN KEY (\`${name}\`) REFERENCES \`${foreing.table}\`(\`${foreing.column}\`)`;
            }
            await this.#get_response(modifyQuery);
          }
        }
      }
      return true;
    } catch (error) {
      console.error("Error al editar columnas.", error);
      throw error;
    }
  }
  /**
   * Deletes columns from the table.
   *
   * @param {Array<string>} fields - An array of column names to delete.
   * @returns {Promise<boolean>} - Returns `true` if the operation was successful.
   *
   * @example
   * await db.table('users').columns().delete(['email']);
   */
  async delete(fields) {
    try {
      const currentFields = await this.get();
      for (const key of fields) {
        if (currentFields[key]) {
          let dropQuery = `ALTER TABLE ${this.tableName} DROP COLUMN \`${key}\`;`;
          await this.#get_response(dropQuery);
        }
      }
      return true;
    } catch (error) {
      throw error;
    }
  }
  async #get_response(sql) {
    const pool = await this.conection;
    if (!pool) {
      throw new Error("No se ha establecido una conexi\xF3n a la base de datos.");
    }
    try {
      const [result] = await pool.query(sql);
      return result;
    } catch (error) {
      if (error.code === "ER_BAD_DB_ERROR") {
        try {
          if (!this.conection) {
            throw new Error("No se ha establecido una conexi\xF3n a la base de datos.");
          }
          await this.conection.end();
          this.conection = mysql.createPool({
            host: MYSQL_HOST,
            user: MYSQL_USER,
            password: MYSQL_PASSWORD,
            port: Number(MYSQL_PORT)
          });
          const pool_aux = await this.conection;
          await pool_aux.query(`CREATE DATABASE \`${MYSQL_DATABASE}\``);
          await this.conection.end();
          this.conection = mysql.createPool({
            host: MYSQL_HOST,
            user: MYSQL_USER,
            password: MYSQL_PASSWORD,
            database: MYSQL_DATABASE,
            port: Number(MYSQL_PORT)
          });
          const pool2 = await this.conection;
          const [result] = await pool2.query(sql);
          return result;
        } catch (error2) {
          throw error2;
        }
      }
      throw error;
    }
  }
};
var db = new MySQL();
var mysql_default = db;

// src/index.ts
var index_default = mysql_default;
export {
  index_default as default
};
//# sourceMappingURL=index.js.map